<?php

/**
 * PSR-4 compatible autoloader for all classes.
 *
 * @author Jarryd Lisher <jarryd@jlisher.com>
 * @license MIT
 */
final class Autoloader
{
    private static bool $registering = false;
    private static bool $registered = false;
    private static self $_instance;
    private static array $pre_files = [];
    private static array $files = [];

    public function __construct()
    {
        if (self::$registered || !self::$registering) {
            return;
        }
        spl_autoload_register(callback: [$this, 'classLoader']);
    }

    private function classLoader(string $class): void
    {
        $path = str_replace(search: '\\', replace: DIRECTORY_SEPARATOR, subject: $class);
        self::requireFile(path: __DIR__ . DIRECTORY_SEPARATOR . $path . '.php');
    }

    /**
     * Instantiate the autoloader and require the any added files.
     */
    public static function register(): void
    {
        if (self::$registering) {
            return;
        }

        self::$registering = true;
        self::requirePreFiles();
        self::$_instance = new self();
        self::$registered = true;
        self::requireFiles();
    }

    /**
     * Add a file to be required pre register.
     *
     * @param string $path
     */
    public static function addPreFile(string $path): void
    {
        if (self::$registering) {
            return;
        }
        self::$pre_files[] = $path;
    }

    /**
     * Add multiple files to be required pre register.
     *
     * @param string[] $paths
     */
    public static function addPreFiles(array $paths): void
    {
        if (self::$registering) {
            return;
        }
        foreach ($paths as $path) {
            self::addPreFile(path: $path);
        }
    }

    private static function requirePreFiles(): void
    {
        while (self::$pre_files) {
            self::requireFile(path: array_shift(array: self::$pre_files));
        }
    }

    /**
     * Add a file to be required post register.
     *
     * @param string $path
     */
    public static function addFile(string $path): void
    {
        if (self::$registered) {
            return;
        }
        self::$files[] = $path;
    }

    /**
     * Add multiple files to be required post register.
     *
     * @param string[] $paths
     */
    public static function addFiles(array $paths): void
    {
        if (self::$registered) {
            return;
        }
        foreach ($paths as $path) {
            self::addFile(path: $path);
        }
    }

    private static function requireFiles(): void
    {
        while (self::$files) {
            self::requireFile(path: array_shift(array: self::$files));
        }
    }

    /**
     * Requires the given path.
     *
     * @param string $path Path of the file to require
     *
     * @return bool Returns true if the file exists and `require_once` was successful.
     */
    public static function requireFile(string $path): bool
    {
        $path = realpath(path: $path);

        if (!$path) {
            return false;
        }

        return (static function (string $path): bool {
            require_once $path;
            return true;
        })(path: $path);
    }
}
